import os
import pandas as pd
from .config import BASE_URL


dirname = os.path.dirname(os.path.abspath(__file__))
df = pd.read_pickle(os.path.join(dirname, 'data.p'))


def get_geo(code, year):
    row = df[df['insee_code'] == code]
    return row.to_dict('records')[0]


def url_resolver(code, year, region_code, department_code):
    zero_pad = lambda num: f'0{str(num)}'

    if year == 2012:
        reg_code = zero_pad(region_code)
        dep_code = zero_pad(department_code)
        com_code = zero_pad(code)
        url = (f'{BASE_URL}elecresult__PR2012/(path)/PR2012/'
               f'{reg_code}/{dep_code}/{com_code}.html')

    return url



from .Commune import Commune
from .utils import df


def list_regions():
    return df[['region', 'region_code']].drop_duplicates()


def list_departments():
    return df[['department', 'department_code',
               'region_code']].drop_duplicates()


def list_communes():
    return df[['commune', 'commune_code', 'insee_code',
               'department_code', 'region_code']].drop_duplicates()


def get(code, year):
    return Commune(code, year)



import pandas as pd
from .utils import url_resolver, get_geo


class Commune():
    "Base class for commune results"
    def __init__(self, code, year):
        self._geo = get_geo(code, year)
        self.url = url_resolver(code, year, self._geo['region_code'],
                                self._geo['department_code'])

        tables = pd.read_html(self.url, header=0, encoding='utf8', decimal=',',
                              thousands=' ')
        self._parse(tables)

    def _parse(self, tables):
        results_round1 = self._parse_results(tables[3], 1)
        results_round2 = self._parse_results(tables[1], 2)
        self.results = pd.concat([results_round1, results_round2])

        meta_round1 = self._parse_meta(tables[2], 1)
        meta_round2 = self._parse_meta(tables[0], 2)
        self.meta = pd.concat([meta_round1, meta_round2])

    def _parse_results(self, table, round_):
        results_cols = {'Liste des candidats': 'candidate', 'Voix': 'votes'}
        table = table.rename(columns=results_cols)
        table['candidate'] = table['candidate'].str.replace('\xa0', ' ')
        table['round'] = round_
        table = table[['round', 'candidate', 'votes']]
        table = table.set_index(['candidate', 'round'])
        return table

    def _parse_meta(self, table, round_):
        clean = lambda x: int(str(x).replace('\xa0', ''))
        table = table[['Unnamed: 0', 'Nombre']].iloc[0:4].set_index('Unnamed: 0').T
        meta_cols = {'Inscrits': 'registered', 'Votants': 'votes',
                     'Blancs ou nuls': 'blanks_or_nulls',
                     'Abstentions': 'abstentions'}
        table = table.rename(columns=meta_cols)
        table['round'] = round_
        table = table.set_index('round')
        table.columns.name = None
        table['registered'] = table['registered'].apply(clean)
        table['votes'] = table['votes'].apply(clean)
        table['blanks_or_nulls'] = table['blanks_or_nulls'].apply(clean)
        table['abstentions'] = table['abstentions'].apply(clean)

        return table

    @property
    def region(self):
        return self._geo['region']

    @property
    def region_code(self):
        return self._geo['region_code']

    @property
    def department(self):
        return self._geo['department']

    @property
    def department_code(self):
        return self._geo['department_code']

    @property
    def name(self):
        return self._geo['commune']

    def __repr__(self):
        return f'<Commune name=\'{self._geo["commune"]}\'>'



from distutils.core import setup

requires = [
    'beautifulsoup4',
    'bs4',
    'html5lib',
    'lxml',
    'numpy',
    'pandas',
    'python-dateutil',
    'pytz',
    'six',
    'webencodings'
]

setup(
    name='frenchy',
    packages=['frenchy'],
    version='0.4',
    description='A pandas friendly french election scraper',
    author='Robin Linderborg',
    author_email='robin.linderborg@gmail.com',
    url='https://github.com/miroli/frenchy',
    download_url='https://github.com/miroli/frenchy/tarball/0.4',
    install_requires=requires,
    keywords=['france', 'scraper', 'elections'],
)
